require 'colored'
require 'json'

module TYCiCore

	# Podfile
	#
	# 关于工程多target约定
	#
	# 1. 多target的Project 取第一个作为主target
	# 2. 修改删除将应用到全部target
	# 3. 对于普通的Project 新增组件将只应用到第一个target
	# 4. 对Pod有封装的target 方法名约定为 pod_install, 新增组件将只应用到该方法
	#

	require 'tuya/ci/core/util/file'

	class Podfile

		PODFILE_TARGET_METHOD = 'pod_install'

		attr_accessor :file
		attr_accessor :content
		attr_accessor :multi_target
		attr_accessor :targets
		attr_accessor :main_target

		def initialize(file='./Podfile')

			@file = file

			puts 'Can not find a Podfile'.red unless File.exist? @file

			@content = File.read @file
			@multi_target = is_multi_target
			@targets = targets
			@main_target = @targets[0][0] if (@targets.size > 0 && @targets[0].size > 0)

		end

		def source_replace(replace_old, replace_new)
			@content.gsub!(/#{replace_old}/, replace_new)
		end

		def source_exist(source)
			TYUtil::TYFile.line_contain_content(@content, source) > 0
		end

		def source_add(source)
			if source.nil? || source.empty?
				puts "source is empty".red
			elsif source_exist(source)
				puts "source #{source} alread exist".yellow
			else
				line = TYUtil::TYFile.line_contain_content(@content, "source")
				TYUtil::TYFile.add_to_line(@content, "source '#{source}'", line)
			end
		end

		def update(updates, deletes)

			puts "Podfile must contain at least one target".red unless @main_target

			if @multi_target
				update_pod_methods(updates, deletes)
			else
				update_single_target(updates, deletes)
			end
		end

		def save

			puts "Podfile: #{@file} saved".green
			puts "Podfile content is: \n #{@content}".green

			fh = File.new(@file, "w")
			fh.puts @content
			fh.close
		end

		private

		def update_pod_methods(updates, deletes)
			update_pods(updates, deletes, "def #{PODFILE_TARGET_METHOD}")
		end

		def update_single_target(updates, deletes)
			update_pods(updates, deletes, "target '#{@main_target}' do")
		end

		def update_pods(updates, deletes, mark_label)

			append_modules = "#{mark_label}\n"
			append_modules = update_update_pods(updates, append_modules) if updates && updates.size > 0

			# deletes.each do |delete|
			# 	json = JSON.parse(JSON(delete))
			# 	name = json["name"]
			# 	@content.gsub!(/\s*pod\s*'#{name}(.*)'$/, "")
			# end

			update_delete_pods deletes if deletes && deletes.size > 0

			@content.gsub!(/^#{mark_label}/, "#{append_modules}\n")
			@content.gsub!(/\n{2,}/, "\n")
		end

		def update_delete_pods(deletes)
			deletes.each do |delete|
				json = JSON.parse(JSON(delete))
				name = json["name"]
				@content.gsub!(/^\s*pod\s*'#{name}'(.*)$/, "")
			end

		end

		def update_update_pods(updates, append_modules)
			updates.each do |update|
				json = JSON.parse(JSON(update))
				name = json["name"]
				version = json["version"]

				module_replace = "\n    pod '#{name}', '#{version}'"
				if @content.scan(/^[^#]\s*pod\s*'#{name}'(.*)'$/).size == 0
					puts "Module #{name} can not find".yellow
					append_modules << module_replace
				else
					puts "Module #{name} got it".green
					if debug? name
						puts "Module #{name} find debug parameter".green
						module_replace = "#{module_replace}, :configuration => 'Debug'"
					else
						puts "Module #{name} did not find debug".yellow
					end
					@content.gsub!(/^[^#]\s*pod\s*'#{name}'(.*)'$/, module_replace)
				end
			end
			append_modules
		end

		def targets
			@content.scan(/target '(\w+)' do/)
		end

		def debug?(name)
			@content.scan(/^[^#]\s*pod\s*'#{name}',(.*,)*\s*:configuration\s*=>\s*'Debug'/).size > 0
		end

		def is_multi_target
			@content.include? PODFILE_TARGET_METHOD
		end

	end

end